<?php

class FindologicItemXml
{
    protected $shopKey;
    protected $languageId;

    /**
     * @var SpxOxarticle
     */
    protected $oArticle;
    protected $oVariantsPrice = array();
    protected $oVariantsOrdNum = array();
    protected $oVariantsId = array();
    protected $userGroups = array();

    /**
     * Builds XML document.
     *
     * @param array $params
     * @param array $products
     * @return string
     */
    public function buildXmlDoc($params, $products)
    {
        $this->shopKey = $params['shopkey'];
        $this->languageId = $params['languageId'];

        $findologic = new SimpleXMLElement('<findologic/>');
        $findologic->addAttribute('version', '1.0');
        $items = $findologic->addChild('items');
        $items->addAttribute('start', $params['start']);
        $items->addAttribute('count', $params['count']);
        $items->addAttribute('total', $params['total']);

        $oxTitle = $this->appendLang('OXTITLE');
        $userGroupSql = 'select ' . $oxTitle . ' from oxgroups';
        $userGroups = oxDb::getDb(oxDb::FETCH_MODE_ASSOC)->getAll($userGroupSql);
        foreach ($userGroups as $group) {
            if ($group[$oxTitle] != 'BLOCKED') {
                $this->userGroups[] = base64_encode($this->shopKey ^ $group[$oxTitle]);
            }
        }

        foreach ($products as $product) {
            $this->oArticle = oxNew('oxarticle');
            $this->oArticle->load($product['OXID']);

            $varSelection = $this->oArticle->getVariants();
            if ($varSelection != null) {
                foreach ($varSelection as $var) {
                    $this->oVariantsPrice[$product['OXID']][] = $var->oxarticles__oxprice->value;
                    $this->oVariantsOrdNum[$product['OXID']][] = array(
                        'OXARTNUM' => $var->oxarticles__oxartnum->value,
                        'OXEAN' => $var->oxarticles__oxean->value,
                        );
                    $this->oVariantsId[$product['OXID']][] = $var->oxarticles__oxid->value;
                }
            }

            $item = $items->addChild('item');
            $item->addAttribute('id', $product['OXID']);
            $this->renderItem($item, $product);
        }

        return $findologic->asXML();
    }

    /**
     * Renders Items for XML Document
     * @param SimpleXMLElement $item
     * @param array $product
     */
    protected function renderItem(SimpleXMLElement $item, $product)
    {
        $this->renderOrderNumber($item->addChild('allOrdernumbers'), $product);

        $this->renderName($item->addChild('names'), $product);

        $this->renderSummary($item->addChild('summaries'), $product);

        $this->renderDescription($item->addChild('descriptions'), $product);

        $this->renderPrice($item->addChild('prices'), $product);

        $this->renderUrl($item->addChild('urls'));

        $this->renderImages($item->addChild('allImages'), $product);

        $this->renderAttributes($item->addChild('allAttributes'), $product);

        $this->renderKeywords($item->addChild('allKeywords'), $product);

        if (!empty($this->userGroups)) {
            $this->renderUserGroups($item->addChild('usergroups'));
        }

        $this->renderSalesFrequency($item->addChild('salesFrequencies'), $product);

        $this->renderDateAdded($item->addChild('dateAddeds'), $product);

        $this->renderSort($item->addChild('sorts'), $product);

        $this->renderAllProperties($item->addChild('allProperties'), $product);
    }

    /**
     * Render order number.
     * @param SimpleXMLElement $allOrderNumbers
     * @param array $product
     */
    protected function renderOrderNumber(SimpleXMLElement $allOrderNumbers, $product)
    {
        $orderNumbers = $allOrderNumbers->addChild('ordernumbers');
        $this->appendCData($orderNumbers->addChild('ordernumber'), $product['OXARTNUM']);
        if ($product['OXEAN']) {
            $this->appendCData($orderNumbers->addChild('ordernumber'), $product['OXEAN']);
        }

        if (!empty($this->oVariantsOrdNum[$product['OXID']])) {
            foreach ($this->oVariantsOrdNum[$product['OXID']] as $variantOrder) {
                $this->appendCData($orderNumbers->addChild('ordernumber'), $variantOrder['OXARTNUM']);
                if ($variantOrder['OXEAN']) {
                    $this->appendCData($orderNumbers->addChild('ordernumber'), $variantOrder['OXEAN']);
                }
            }
        }
    }

    /**
     * Render name in XML
     * @param SimpleXMLElement $names
     * @param array $product
     */
    protected function renderName(SimpleXMLElement $names, $product)
    {
        $this->appendCData($names->addChild('name'), $product[$this->appendLang('OXTITLE')]);
    }

    /**
     * Render summary in XML
     * @param SimpleXMLElement $summaries
     * @param array $product
     */
    protected function renderSummary(SimpleXMLElement $summaries, $product)
    {
        $this->appendCData($summaries->addChild('summary'), $product[$this->appendLang('OXSHORTDESC')]);
    }

    /**
     * Render Description in XML
     * @param SimpleXMLElement $descriptions
     * @param array $product
     */
    protected function renderDescription(SimpleXMLElement $descriptions, $product)
    {
        $this->appendCData($descriptions->addChild('description'), $product[$this->appendLang('OXLONGDESC')]);
    }

    /**
     * Render price in XML
     * @param SimpleXMLElement $prices
     * @param array $product
     */
    protected function renderPrice(SimpleXMLElement $prices, $product)
    {
        $specVAT = $product['OXVAT']; // null
        $catVat = $product['catVat'];

        $defaultVat = oxRegistry::getConfig()->getShopConfVar('dDefaultVAT');
        $enterNetPrice = oxRegistry::getConfig()->getShopConfVar('blEnterNetPrice'); // without vat if true
        $showNetPrice = oxRegistry::getConfig()->getShopConfVar('blShowNetPrice'); // without vat if true
        // find lowest price from all variants if any
        if (!empty($this->oVariantsPrice[$product['OXID']])) {
            $productPrice = $product['OXPRICE'];
            foreach ($this->oVariantsPrice[$product['OXID']] as $varPrice) {
                if ($varPrice < $productPrice) {
                    $productPrice = $varPrice;
                }
            }
        } else {
            $productPrice = $product['OXPRICE'];
        }

        if ($enterNetPrice) {
            if ($showNetPrice) {
                $price = $productPrice;
            } else {
                if (($specVAT != null) && ($specVAT != '')) {
                    $price = $productPrice * ( 1 + $specVAT / 100);
                } else if (($catVat != null) && ($catVat != '')) {
                    $price = $productPrice * ( 1 + $catVat / 100);
                } else {
                    $price = $productPrice * ( 1 + $defaultVat / 100);
                }
            }
        } else {
            if ($showNetPrice) {
                if (($specVAT != null) && ($specVAT != '')) {
                    $price = $productPrice / ( 1 + $specVAT / 100);
                } else if (($catVat != null) && ($catVat != '')) {
                    $price = $productPrice / ( 1 + $catVat / 100);
                } else {
                    $price = $productPrice / ( 1 + $defaultVat / 100);
                }
            } else {
                $price = $productPrice;
            }
        }

        $this->appendCData($prices->addChild('price'), sprintf('%.2f', $price));

        if (($product['OXPRICEA'] != '0') && ($product['OXPRICEA'] != null)) {
            $priceANode = $prices->addChild('price');
            $priceANode->addAttribute('usergroup', base64_encode($this->shopKey ^ 'Price A'));
            $this->appendCData($priceANode, sprintf('%.2f', $product['OXPRICEA']));
        }
        if (($product['OXPRICEB'] != '0') && ($product['OXPRICEB'] != null)) {
            $priceBNode = $prices->addChild('price');
            $priceBNode->addAttribute('usergroup', base64_encode($this->shopKey ^ 'Price B'));
            $this->appendCData($priceBNode, sprintf('%.2f', $product['OXPRICEB']));
        }
        if (($product['OXPRICEC'] != '0') && ($product['OXPRICEC'] != null)) {
            $priceCNode = $prices->addChild('price');
            $priceCNode->addAttribute('usergroup', base64_encode($this->shopKey ^ 'Price C'));
            $this->appendCData($priceCNode, sprintf('%.2f', $product['OXPRICEC']));
        }
    }

    /**
     * Render URL
     * @param SimpleXMLElement $urls
     */
    protected function renderUrl(SimpleXMLElement $urls)
    {
        $this->appendCData($urls->addChild('url'), $this->oArticle->getLink($this->languageId));
    }

    /**
     * Render Images
     * @param SimpleXMLElement $allImages
     * @param array $product
     */
    protected function renderImages(SimpleXMLElement $allImages, $product)
    {
        $allImagesToR = array();
        $imageGallery = $this->oArticle->getPictureGallery();
        $thumbnail = $this->oArticle->getThumbnailUrl();
        $images = $allImages->addChild('images');
        if ($thumbnail) {
            $thumbnailNode = $images->addChild('image');
            $this->appendCData($thumbnailNode, $thumbnail);
            $thumbnailNode->addAttribute('type', 'thumbnail');
        }

        if (!empty($imageGallery)) {
            // check if article have images
            foreach ($imageGallery['Pics'] as $imag) {
                $allImagesToR[] = $imag;
            }
            foreach ($imageGallery['Icons'] as $imag) {
                $allImagesToR[] = $imag;
            }
            foreach ($imageGallery['ZoomPics'] as $imag) {
                $allImagesToR[] = $imag['file'];
            }
            if (!empty($this->oVariantsId[$product['OXID']])) {

                foreach ($this->oVariantsId[$product['OXID']] as $variant) {
                    $variantArt = oxNew('oxarticle');
                    $variantArt->load($variant);
                    $imageGal = $variantArt->getPictureGallery();

                    if (!empty($imageGal)) {
                        foreach ($imageGal['Pics'] as $imag) {
                            $allImagesToR[] = $imag;
                        }
                        foreach ($imageGal['Icons'] as $imag) {
                            $allImagesToR[] = $imag;
                        }
                        foreach ($imageGal['ZoomPics'] as $imag) {
                            $allImagesToR[] = $imag['file'];
                        }
                    }
                }
            }
            $imagesToRender = array_unique($allImagesToR);

            foreach ($imagesToRender as $imageToRender) {
                $this->appendCData($images->addChild('image'), $imageToRender);
            }
        } else {
            $this->appendCData($images->addChild('image'), oxRegistry::getConfig()->getPictureUrl(1));
        }
    }

    /**
     * Exports all categories.
     *
     * @param int $languageId
     * @param int $productId
     * @return array
     */
    protected function exportAllCategories($languageId, $productId)
    {
        $articleID = $productId;
        $article = oxNew('oxArticle');
        $article->load($articleID);
        $cat = array();
        $catIDs = $article->getCategoryIds();
        $allCategories = array();
        $oxConfig = oxNew('oxConfig');
        foreach ($catIDs as $catID) {
            $baseURL = $oxConfig->getShopUrl();
            /** @var oxCategory $oCategory */
            $oCategory = oxNew('oxCategory');
            $oCategory->load($catID);
            $catNames = array($oCategory->getTitle());
            $cat['url'][] = '/' . str_replace($baseURL, '', $oCategory->getLink($languageId));
            $allCategories[$oCategory->getId()] = $oCategory->getTitle();
            while ($oCategory = $oCategory->getParentCategory()) {
                $allCategories[$oCategory->getId()] = $oCategory->getTitle();
                $catNames[] = $oCategory->getTitle();
            }

            $cat['cat'][] = html_entity_decode(implode('_', array_reverse($catNames)));
        }

        $cat['titles'] = array_reverse($allCategories);
        return $cat;
    }

    /**
     * Render Attributes
     * @param SimpleXMLElement $allAttributes
     * @param array $product
     */
    protected function renderAttributes(SimpleXMLElement $allAttributes, $product)
    {
        $categories = $this->exportAllCategories($this->languageId, $product['OXID']);
        $attributesNode = $allAttributes->addChild('attributes');

        // category
        $catNode = $attributesNode->addChild('attribute');

        $this->appendCData($catNode->addChild('key'), 'cat');
        $valuesNode = $catNode->addChild('values');
        foreach ($categories['cat'] as $category) {
            $this->appendCData($valuesNode->addChild('value'), $category);
        }


        // category url
        $attributeCatUrl = $attributesNode->addChild('attribute');
        $this->appendCData($attributeCatUrl->addChild('key'), 'cat_url');
        $valuesCatUrl = $attributeCatUrl->addChild('values');
        foreach ($categories['url'] as $url) {
            $this->appendCData($valuesCatUrl->addChild('value'), $url);
        }

        // add manufacturer
        if (!empty($product['manufacturerName']) || $product['manufacturerName'] != '') {
            $manufacturerNode = $attributesNode->addChild('attribute');
            $this->appendCData($manufacturerNode->addChild('key'), 'brand');
            $manufacturerValuesNode = $manufacturerNode->addChild('values');
            $this->appendCData($manufacturerValuesNode->addChild('value'), $product['manufacturerName']);
        }
        // add variants
        $varSelection = $this->oArticle->getVariantSelections();
        $varSelect = $varSelection['selections'];

        if ($varSelect != null) {

            foreach ($varSelect as $var) {
                $variantNode = $attributesNode->addChild('attribute');
                $this->appendCData($variantNode->addChild('key'), $var->getLabel());
                $valuesNode = $variantNode->addChild('values');
                foreach ($var->getSelections() as $varSel) {
                    $this->appendCData($valuesNode->addChild('value'), $varSel->getName());
                }
            }
        }

        // votes rating
        if (!empty($product['OXRATING'])) {
            $votesRatingNode = $attributesNode->addChild('attribute');
            $this->appendCData($votesRatingNode->addChild('key'), 'votes_rating');
            $votesRatingValuesNode = $votesRatingNode->addChild('values');
            $this->appendCData($votesRatingValuesNode->addChild('value'), round($product['OXRATING']));
        }

        // sale
        if (!empty($product['OXPRICE'])) {
            $saleNode = $attributesNode->addChild('attribute');
            $this->appendCData($saleNode->addChild('key'), 'sale');
            $saleNodeValuesNode = $saleNode->addChild('values');
            $isOnSale = $product['OXPRICE'] < $product['OXTPRICE'] ? 1 : 0;
            $this->appendCData($saleNodeValuesNode->addChild('value'), $isOnSale);
        }

        // availability
        $availability = $this->getValueIfKeyExists('OXSTOCK', $product);
        if (!empty($availability)) {
            $availabilityNode = $attributesNode->addChild('attribute');
            $this->appendCData($availabilityNode->addChild('key'), 'availability');
            $availabilityValuesNode = $availabilityNode->addChild('values');
            $this->appendCData($availabilityValuesNode->addChild('value'),
                !empty($availability) ? $this->getAvailabilityMessage($product) : ''
            );
        }
    }

    /**
     * Render Keywords
     * @param SimpleXMLElement $allKeywords
     * @param array $product
     */
    protected function renderKeywords(SimpleXMLElement $allKeywords, $product)
    {
        $oxSearchKeys = $this->appendLang('OXSEARCHKEYS');
        if (count($product[$oxSearchKeys]) > 0) {
            $meteKeywords = explode(',', $product[$oxSearchKeys]);
            $keywords = $allKeywords->addChild('keywords');

            foreach ($meteKeywords as $keyword) {
                $keyword = trim($keyword);
                if ($keyword !== '') {
                    $this->appendCData($keywords->addChild('keyword'), $keyword);
                }
            }
        }
    }

    /**
     * Render UserGroups
     * @param SimpleXMLElement $usergroups
     */
    protected function renderUserGroups(SimpleXMLElement $usergroups)
    {
        foreach ($this->userGroups as $group) {
            $this->appendCData($usergroups->addChild('usergroup'), $group);
        }
    }

    /**
     * Render Sales Frequency
     * @param SimpleXMLElement $salesFrequencies
     * @param array $product
     */
    protected function renderSalesFrequency(SimpleXMLElement $salesFrequencies, $product)
    {
        $salesFrequenciesCount = $product['salesFrequency'];

        if ($salesFrequenciesCount > 0) {
            $this->appendCData($salesFrequencies->addChild('salesFrequency'), $salesFrequenciesCount);
        }
    }

    /**
     * Randar Date Added
     * @param SimpleXMLElement $dateAdded
     * @param array $product
     */
    protected function renderDateAdded(SimpleXMLElement $dateAdded, $product)
    {
        $format = 'Y-m-d';
        /* @var $date DateTime */
        $date = DateTime::createFromFormat($format, $product['OXINSERT']);

        $this->appendCData($dateAdded->addChild('dateAdded'), $date->format(DATE_ATOM));
    }

    /**
     * Render Sort
     * @param SimpleXMLElement $sorts
     * @param array $product
     */
    protected function renderSort(SimpleXMLElement $sorts, $product)
    {
        if ($product['OXSORT'] != 0) {
            $this->appendCData($sorts->addChild('sort'), $product['OXSORT']);
        }
    }

    /**
     * Render All properties
     * @param SimpleXMLElement $allProperties
     * @param array $product
     */
    protected function renderAllProperties(SimpleXMLElement $allProperties, $product)
    {
        $properties = $allProperties->addChild('properties');
        $defaultVat = oxRegistry::getConfig()->getShopConfVar('dDefaultVAT');
        $availability = $this->getValueIfKeyExists('OXSTOCK', $product);
        $tax = $this->getValueIfKeyExists('OXVAT', $product);
        $oxPrice = $this->getValueIfKeyExists('OXPRICE', $product);
        $oxTPrice = $this->getValueIfKeyExists('OXTPRICE', $product);
        $oxVarCount = $this->getValueIfKeyExists('OXVARCOUNT', $product);
        $oxVarMinPrice = $this->getValueIfKeyExists('OXVARMINPRICE', $product);
        $oxVarMaxPrice = $this->getValueIfKeyExists('OXVARMAXPRICE', $product);

        $productProperties = array(
            'shippingfree' => $this->getValueIfKeyExists('OXFREESHIPPING', $product),
            'votes_rating' => $this->getValueIfKeyExists('OXRATING', $product),
            'votes_count' => $this->getValueIfKeyExists('OXRATINGCNT', $product),
            'unit' => $this->getValueIfKeyExists('OXUNITNAME', $product),
            'unit_quantity' => $this->getValueIfKeyExists('OXUNITQUANTITY', $product),
            'min_delivery_time' => $this->getValueIfKeyExists('OXMINDELTIME', $product)
                . ' '
                . $this->getValueIfKeyExists('OXDELTIMEUNIT', $product),
            'max_delivery_time' => $this->getValueIfKeyExists('OXMAXDELTIME', $product)
                . ' '
                . $this->getValueIfKeyExists('OXDELTIMEUNIT', $product),
            'weight' => $this->getValueIfKeyExists('OXWEIGHT', $product),
            'width' => $this->getValueIfKeyExists('OXWIDTH', $product),
            'height' => $this->getValueIfKeyExists('OXHEIGHT', $product),
            'length' => $this->getValueIfKeyExists('OXLENGTH', $product),
            'availability' => !empty($availability) ? $this->getAvailabilityMessage($product) : '',
            'tax' => $tax !== '' && is_null($tax) ? $defaultVat : $tax,
            'uvp' => $this->getValueIfKeyExists('OXTPRICE', $product),
            'quantity' => $this->getValueIfKeyExists('OXSTOCK', $product),
        );

        if ((!empty($oxPrice) && !empty($oxTPrice)) && ($oxPrice < $oxTPrice)) {
            $productProperties['old_price'] = $oxTPrice;
        }

        $additionalProductProperties = array(
            'has_variants' => !empty($oxVarCount) && $oxVarCount > 0 ? 1 : 0,
        );

        if ($additionalProductProperties['has_variants'] == 1) {
            $oxVarDiffPrice = 0;
            if ((!empty($oxVarMinPrice) && !empty($oxVarMaxPrice)) && $oxVarMinPrice == $oxVarMaxPrice) {
                $oxVarDiffPrice = 1;
            }

            $additionalProductProperties['show_from_price'] = $oxVarDiffPrice;
        }

        $productProperties = $this->addCustomProperties($productProperties, $product);

        foreach ($productProperties as $key => $propertyValue) {
            if (!is_null($propertyValue) && !empty($propertyValue) && $propertyValue !== 0) {
                $property = $properties->addChild('property');
                $this->appendCData($property->addChild('key'), $key);
                $this->appendCData($property->addChild('value'), $propertyValue);
            }
        }

        foreach ($additionalProductProperties as $key => $propertyValue) {
            $property = $properties->addChild('property');
            $this->appendCData($property->addChild('key'), $key);
            $this->appendCData($property->addChild('value'), $propertyValue);
        }
    }

    /**
     * Adds custom properties. To be used in child class.
     *
     * @param array $productProperties
     * @return array
     */
    protected function addCustomProperties($productProperties, $product)
    {
        return $productProperties;
    }

    /**
     * Gets proper availability message according to languageId
     *
     * @param array $product
     */
    private function getAvailabilityMessage($product)
    {
        $oLang = oxNew('oxLang');

        if ((int)$product['OXSTOCK'] === (int)0) {
            $notInStock = $oLang->getSimilarByKey('MESSAGE_NOT_ON_STOCK', $this->languageId);
            return $notInStock['MESSAGE_NOT_ON_STOCK'];
        } else if ((int)$product['OXSTOCK'] === (int)1) {
            $lowStock = $oLang->getSimilarByKey('LOW_STOCK', $this->languageId);
            return $lowStock['LOW_STOCK'];
        } else {
            $readyForShipping = $oLang->getSimilarByKey('READY_FOR_SHIPPING', $this->languageId);
            return $readyForShipping['READY_FOR_SHIPPING'];
        }
    }

    /**
     * Checks if key exists in given array
     *
     * @param string $key
     * @param array $product
     * @return string
     */
    private function getValueIfKeyExists($key, $product) {
        return array_key_exists($key, $product) ? $product[$key] : '';
    }

    /**
     * Append Cdata to element
     * @param SimpleXMLElement $node
     * @param mixed $text
     */
    protected function appendCData(SimpleXMLElement $node, $text)
    {
        $domNode = dom_import_simplexml($node);
        $domNode->appendChild($domNode->ownerDocument->createCDATASection($this->utf8Replace($text)));
    }

    protected function utf8Replace($text)
    {
        return preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x80-\x9F]/u', '', trim($text));
    }

    /**
     * Appends language id only if non default language is selected for data export.
     *
     * @param string $input
     * @return string
     */
    protected function appendLang($input)
    {
        $lang = $this->languageId > 0 ? '_' . $this->languageId : '';

        return $input . $lang;
    }
}
