<?php

class Findologic extends oxUBase
{

    /**
     * @var string
     */
    private $shopKey;

    /**
     * @var integer
     */
    private $storeId;

    /**
     * @var integer
     */
    private $count;

    /**
     * @var integer
     */
    private $start;

    /**
     * @var integer
     */
    private $total;

    /**
     * @var integer
     */
    private $languageId;

    public function init()
    {
        $this->shopKey = oxRegistry::getConfig()->getRequestParameter('shopkey');
        $this->start = oxRegistry::getConfig()->getRequestParameter('start');
        $this->count = oxRegistry::getConfig()->getRequestParameter('count');

        $configShopKey = $this->checkShopKey($this->shopKey);

        $this->storeId = $this->getStore($this->shopKey, $configShopKey);
        $this->validateInput();

        $products = $this->getValidArticles($this->start, $this->count, $this->storeId);

        // prevent generateItemNodes function call for non-existing articles
        if (((int) $this->start + (int) $this->count) > $this->total) {
            $this->count = $this->total - (int) $this->start;
        }


        $xml = $this->createXml($products);

        if (oxRegistry::getConfig()->getRequestParameter('validate', false)) {
            $this->validateXml($xml);
        }

        header('Content-Type: application/xml; charset=utf-8');
        echo $xml;
        die;
    }

    /**
     * Validates whether all input parameters are supplied and valid.
     */
    private function validateInput()
    {
        $message = '';
        if (!$this->shopKey) {
            $message = 'Parameter "shopkey" is missing! ';
        }

        if (!$this->storeId) {
            $message .= 'Parameter "shopkey" is not configured for any store! ';
        }

        if ($this->start === false || $this->start < 0) {
            $message .= 'Parameter "start" is missing or less than 0! ';
        }

        if (!$this->count || $this->count < 1) {
            $message .= 'Parameter "count" is missing or less than 1!';
        }
        
        if (!is_numeric($this->start) || !is_numeric($this->count)) {
            $message .= 'Parameters must be numeric values.<br> Please provide start and count values in precise format';  
        }

        if (strpos($this->start, '.') || strpos($this->count, '.')) {
            $message .= 'Parameters must be integer values.<br> Please provide start and count values in precise format';
        }

        if ($message) {
            die($message);
        }
    }

    /**
     * Check if there is such shopkey, grab shopId registered with this shop
     * @param string $reqShopKey
     * @param string $configShopKey
     * @return int
     */
    private function getStore($reqShopKey, $configShopKey)
    {
        if ($reqShopKey != $configShopKey) {
            echo 'There is no shop attached to this key';
            die;
        }

        $storeId = oxDb::getDb()->getOne('SELECT OXID FROM oxshops');
        if (!$storeId) {
            return $this->getConfig()->getActiveShop()->getId();
        }

        return $storeId;
    }

    /**
     * 
     * @param int $start
     * @param int $count
     * @param int $shopID
     * @return array
     */
    private function getValidArticles($start, $count, $shopID)
    {
        $countSql = 'SELECT count(*)'
            . 'FROM oxarticles where oxarticles.OXSHOPID="'
            . $shopID . '" and oxarticles.OXACTIVE=1 and oxarticles.OXSTOCK>0 and oxarticles.OXPARENTID=' . "''";
        $this->total = oxDb::getDb()->getOne($countSql);

        $lang = $this->languageId > 0 ? '_' . $this->languageId : '';
        // articles with parentId are variants !!
        $articlesSql =
            "select oxarticles.* , oxartextends.OXLONGDESC$lang, o2c.OXCATNID ,
            oxcategories.OXTITLE$lang as catTitle,
            ifnull(oxcategories.OXVAT,0) as catVat,
            oxmanufacturers.OXTITLE$lang as manufacturerName,
            (
                /*
                if article have many categories oxid defaults first
                entered for search, otherwise returns url from the category
                */
                select oxseo.OXSEOURL from oxseo
                where oxseo.OXOBJECTID = oxarticles.OXID
                and oxseo.OXLANG = $this->languageId
                order by OXTIMESTAMP
                limit 1
            )    OXSEOURL, ifnull(t.salesFrequency,0) as salesFrequency
            from oxarticles
            left join oxartextends
                on oxarticles.OXID = oxartextends.OXID
            left join oxobject2category as o2c
                on oxarticles.OXID = o2c.OXOBJECTID
            left join oxcategories
                on o2c.OXCATNID = oxcategories.OXID
            left join oxmanufacturers
                on oxarticles.OXMANUFACTURERID = oxmanufacturers.OXID
            left join
                (
                    SELECT OXARTID, count(*) as salesFrequency FROM oxorderarticles
                    group by OXARTID
                ) t
                on t.OXARTID = oxarticles.OXID
            where oxarticles.OXSHOPID='$shopID'
            and oxarticles.OXACTIVE=1
            and oxarticles.OXSTOCK>0
            and oxarticles.OXPARENTID=''
            and oxcategories.OXACTIVE = 1
            and oxcategories.OXHIDDEN = 0
            GROUP BY oxarticles.OXID LIMIT  $start, $count";

        $articles = oxDb::getDb(oxDb::FETCH_MODE_ASSOC)->getAll($articlesSql);

        return $articles;
    }

    /**
     * Creates XML file.
     *
     * @param array $products
     * @return string
     */
    private function createXml($products)
    {
        $customExportFilePath = oxRegistry::getConfig()->getConfigParam('sShopDir') . 'modules/findologic/findologic/findologicCustomExport.php';

        if (file_exists($customExportFilePath)) {
            require_once $customExportFilePath;
            $xml = new FindologicCustomExport();
        } else {
            $xml = new FindologicItemXml();
        }

        return $xml->buildXmlDoc(
            array(
                'shopkey' => $this->shopKey,
                'count' => $this->count,
                'start' => $this->start,
                'total' => $this->total,
                'languageId' => $this->languageId,
            ),
            $products
        );
    }

    /**
     * Validates XML
     * @param string $xml
     */
    private function validateXml($xml)
    {
        $ds = DIRECTORY_SEPARATOR;
        // Enable user error handling
        libxml_use_internal_errors(true);

        $dom = new DOMDocument();
        $dom->loadXML($xml);
        if (!$dom->schemaValidate(realpath(dirname(__FILE__) . $ds . '..' . $ds . 'lib' . $ds . 'findologic.xsd'))) {
            $errors = libxml_get_errors();
            foreach ($errors as $error) {
                $return = "<br/>\n";
                switch ($error->level) {
                    case LIBXML_ERR_WARNING:
                        $return .= "<b>Warning $error->code</b>: ";
                        break;
                    case LIBXML_ERR_ERROR:
                        $return .= "<b>Error $error->code</b>: ";
                        break;
                    case LIBXML_ERR_FATAL:
                        $return .= "<b>Fatal Error $error->code</b>: ";
                        break;
                }

                $return .= trim($error->message);
                if ($error->file) {
                    $return .= " in <b>$error->file</b>";
                }

                echo $return . " on line <b>$error->line</b>\n";
            }

            die;
        }
    }

    /**
     * Checks if shopkey exists
     *
     * @param type string $shopKey
     * @return string
     */
    private function checkShopKey($shopKey)
    {
        $languages = oxNew('oxLang')->getLanguageArray();

        foreach($languages as $language) {
            $confShopKeys[$language->id] = $this->getConfig()->getConfigParam('FindologicShopkey_' . $language->id);
            if ($confShopKeys[$language->id] === $shopKey) {
                $this->languageId = $language->id;

                return $shopKey;
            }
        }

        return false;
    }
}
